// based on example from http://en.cppreference.com/w/cpp/numeric/random/rand
#include <cstdlib>
#include <iostream>
#include <ctime>
#include <random>
#include <list>
#include <algorithm>
#include <unordered_map>
#include <set>
#include <map>
#include <vector>
#include "grundgeruest.hpp"
#include "BinaryOctet.h"

// for println();
std::string as_string(BinaryOctet a) {
    std::string result = "(";
    for (char c : a.bitsAsDigits) {
        result += c;
    }
    result += ")";
    return result;
}

std::string as_string(BinaryOctet *a) {
    std::string result = "(";
    for (char c : a->bitsAsDigits) {
        result += c;
    }
    result += ")";
    return result;
}

// for std::cout
std::ostream &operator<<(std::ostream &os, const BinaryOctet &toBePrinted) {
    os << as_string(toBePrinted);
    return os;
}


std::ostream &operator<<(std::ostream &os, BinaryOctet *toBePrinted) {
    os << as_string(toBePrinted);
    return os;
}


int main() {
    std::srand(static_cast<unsigned int>(std::time(nullptr)));
    int array[41];

    std::default_random_engine generator;
    std::uniform_int_distribution<int> distribution(0, 23);

    for (int i = 0; i <= 41; i++) {
        int rnd = distribution(generator);
        array[i] = rnd;
    }

    unittest_stringSimilarity();

    std::vector<int> intVector;
    for (int i = 0; i <= 41; i++) {
        int rnd = distribution(generator);
        auto found = std::find(intVector.begin(), intVector.end(), rnd);
        if(found == intVector.end()) {
            intVector.push_back(rnd);
        }
        std::sort(intVector.begin(), intVector.end());
    }
    for (int ai : intVector) {
        std::cout << ai << ",";
    }
    std::cout << std::endl;

    std::vector<BinaryOctet> binVector;
    for (int j = 0; j < 41; ++j) {
        int rnd = distribution(generator);
        auto binaryOctet = BinaryOctet(rnd);
        auto found = std::find(binVector.begin(), binVector.end(), binaryOctet);
        if(found == binVector.end()) {
            binVector.push_back(binaryOctet);
        }
        std::sort(binVector.begin(), binVector.end());
    }
    for (BinaryOctet bin : binVector) {
        std::cout << bin << ",";
    }

}

//TODO: mehr Punkte verteilen
//TODO: Punkte gleichmässig verteilen
int stringSimilarity(std::string string1, std::string string2) {
    //wenn gleicher String, gleich 100 Punkte geben, spart den Rest
    if (string1 == string2) return 100;
    int points = 0;

    //gleiche Länge +10 Punkte
    if (string1.length() == string2.length()) points += 10;

    std::map<char, int> charCount;
    std::map<char, int> charCountString1;
    std::map<char, int> charCountString2;
    for (char c : string1) {
        charCount[c]++;
        charCountString1[c]++;
    }
    for (char c : string2) {
        charCount[c]++;
        charCountString2[c]++;
    }
    std::cout << charCount.size() << " : " << charCountString1.size() << " : " << charCountString2.size() << std::endl;
    int equalCharsPercentage = static_cast<int>((double(charCountString1.size()) / double(charCount.size())) * 100);

    points += equalCharsPercentage / 10;



    return points;
}

void unittest_stringSimilarity() {
    std::cout << "Hallo Welt | Hallo Welt ---> " << stringSimilarity("Hallo Welt", "Hallo Welt") << std::endl;
    std::cout << "Hallo Welt | Hallo Baum --->" << stringSimilarity("Halllo Welt", "Hallo Baum") << std::endl;
    std::cout << "Hello World | Hallo Welt ---> " << stringSimilarity("Hello World", "Hallo Welt") << std::endl;
    std::cout << "bla | ping ---> " << stringSimilarity("bla", "ping") << std::endl;
    std::cout << "Lorem ipsum..... | Lorem ipsum ---> " << stringSimilarity("Lorem ipsum dolor sit amet, "
                                                                            "consetetur sadipscing elitr, "
                                                                            "sed diam nonumy eirmod tempor"
                                                                            "invidunt ut labore et dolore "
                                                                            "magna aliquyam erat, sed diam voluptua. "
                                                                            "At vero eos et accusam et justo duo dolores "
                                                                            "et ea rebum. Stet clita kasd gubergren, "
                                                                            "no sea takimata sanctus est Lorem "
                                                                            "ipsum dolor sit amet. "
                                                                            "Lorem ipsum dolor sit amet, "
                                                                            "consetetur sadipscing elitr, "
                                                                            "sed diam nonumy eirmod tempor "
                                                                            "invidunt ut labore et dolore magna "
                                                                            "aliquyam erat, sed diam voluptua. "
                                                                            "At vero eos et accusam et justo duo dolores "
                                                                            "et ea rebum. Stet clita kasd gubergren, "
                                                                            "no sea takimata sanctus est Lorem ipsum "
                                                                            "dolor sit amet.", "Lorem ipsum")
              << std::endl;
}