#include <iostream>
#include <algorithm>
#include <math.h>
#include "BinaryOctet.h"


BinaryOctet::BinaryOctet(int x) {
    for (char &bitsAsDigit : bitsAsDigits) {
        bitsAsDigit = '0';
    }

    int remainder;
    int temp = 0;
    while (x != 0) {
        remainder = x % 2;
        x /= 2;
        bitsAsDigits[temp] = static_cast<char>(remainder + '0');
        temp++;
    }

    std::reverse(std::begin(bitsAsDigits), std::end(bitsAsDigits));

    int ones = 0;
    for (char c : bitsAsDigits) {
        if (c == '1') ones++;
    }

    evenParity = ones % 2 == 0;
}

const BinaryOctet BinaryOctet::operator--(int) {
    return operator-(*new BinaryOctet(1));
}

const BinaryOctet BinaryOctet::operator++(int) {
    return operator+(*new BinaryOctet(1));
}

BinaryOctet BinaryOctet::operator+(BinaryOctet a) {
    BinaryOctet result;
    bool over = false;
    for (int i = bitsPerOctet - 1; i >= 0; --i) {
        //Arbeitsvariablen anlegen und char in int umwandeln für einfachere Rechnung
        int tmp;
        int aI = a.bitsAsDigits[i] - '0';
        int bI = bitsAsDigits[i] - '0';


        if(over) tmp = aI + bI + 1;
        else tmp = aI + bI;
        over = false;


        if(tmp >= 2) {
            over = true;
            if (tmp % 2 == 0) tmp = 0;
            else tmp = 1;
        }

        //Umwandlung in char und ins Ergebnis schreiben
        result.bitsAsDigits[i] = static_cast<char>(tmp + '0');
    }
    return result;
}

BinaryOctet BinaryOctet::operator-(BinaryOctet a) {
    BinaryOctet result;
    bool over = false;
    for(int i = bitsPerOctet - 1; i >= 0; --i) {
        int tmp;
        int aI = a.bitsAsDigits[i] - '0';
        int bI = bitsAsDigits[i] - '0';

        if(over) tmp = bI - aI - 1;
        else tmp = bI - aI;
        over = false;

        if(tmp < 0) {
            over = true;
            tmp = tmp + 1;
        }
        result.bitsAsDigits[i] = static_cast<char>(tmp + '0');
    }
    return result;
}

BinaryOctet BinaryOctet::operator/(BinaryOctet &a) {
    BinaryOctet result = *new BinaryOctet(0);

    while(*this >= a) {
        *this = *this - a;
        result = result + BinaryOctet(1);
    }
    return result;
}

bool BinaryOctet::operator==(const BinaryOctet &a) {
    if(a.evenParity != evenParity) return false;
    for (int i = 0; i < bitsPerOctet; ++i) {
        if(a.bitsAsDigits[i] != bitsAsDigits[i]) return false;
    }
    return true;
}

bool BinaryOctet::operator!=(BinaryOctet &a) {
    if (a.evenParity !=evenParity) return true;
    for (int i = 0; i < bitsPerOctet; ++i) {
        if (a.bitsAsDigits[i] != bitsAsDigits[i]) return true;
    }
    return false;
}

bool BinaryOctet::operator>=(BinaryOctet &a) {

    int power = 8;
    int aI = 0;
    for(int i = 0; i <= bitsPerOctet - 1; i++) {
        power--;
        aI = aI + (a.bitsAsDigits[i] - '0') * pow(2, power);
    }

    power = 8;
    int bI = 0;
    for(int i = 0; i <= bitsPerOctet - 1; i++) {
        power--;
        bI = bI + (bitsAsDigits[i] - '0') * pow(2, power);
    }
    return bI >= aI;
}

bool BinaryOctet::operator<(BinaryOctet &a) {

    int power = 8;
    int aI = 0;
    for(int i = 0; i <= bitsPerOctet - 1; i++) {
        power--;
        aI = aI + (a.bitsAsDigits[i] - '0') * pow(2, power);
    }

    power = 8;
    int bI = 0;
    for(int i = 0; i <= bitsPerOctet - 1; i++) {
        power--;
        bI = bI + (bitsAsDigits[i] - '0') * pow(2, power);
    }
    return bI < aI;
}
