#include <set>
#include <algorithm>
#include <iostream>
#include <ctime>
#include <random>
#include "Currency.h"
#include "BankAccount.h"
#include "Bill.h"
#include "CashFactory.h"
#include "Bank.h"

std::default_random_engine generator(static_cast<unsigned long>(time(nullptr)));
std::uniform_int_distribution<int> distribution(-100, 500);

std::string currency_to_string(CurrencyValue value) {
    if(value == CurrencyValue::USD) return "US Dollar";
    if(value == CurrencyValue::EUR) return "Euro";
    if(value == CurrencyValue::GPD) return "British Pound";
    return "Say what ?";
}

std::ostream &operator<<(std::ostream &os, Currency currency) {
    os << currency_to_string(currency.getValue());
    return os;
}

std::ostream &operator<<(std::ostream &os, Bill bill) {
    os << bill.getValue() << " " << bill.getCurrency() << " " << bill.getSerial();
    return os;
}

std::ostream &operator<<(std::ostream &os, Cash cash) {
    os << cash.getValue() << " " << cash.getCurrency();
    return os;
}

std::ostream &operator<<(std::ostream &os, Money money) {
    os << money.getValue() << " " << money.getCurrency();
    return os;
}

std::ostream &operator<<(std::ostream &os, Person * person) {
    os << person->getName();
    return os;
}

std::ostream &operator<<(std::ostream &os, BankAccount bankAccount) {
    os << bankAccount.getOwner() << " " << bankAccount.getName() << ": " << bankAccount.getMoney();
    return os;
}


int main(int argc, char **argv) {
    std::set<Bank*> banks;

    Bank * bank1 = new Bank("Bank 1");
    Bank * bank2 = new Bank("Bank 2");
    Bank * bank3 = new Bank("Bank 3");

    Person * max = new Person("Max Müller");
    Person * marius = new Person("Marius Müller");
    Person * alex = new Person("Alex Maier");


    bank1->addAccount(new BankAccount(max, "Konto 1"));
    bank2->addAccount(new BankAccount(max, "Konto 2"));
    bank1->addAccount(new BankAccount(alex, "privat"));
    bank3->addAccount(new BankAccount(alex, "geschäftlich 1"));
    bank2->addAccount(new BankAccount(alex, "geschäftlich 2"));
    bank1->addAccount(new BankAccount(alex, "geschäftlich 3"));
    bank2->addAccount(new BankAccount(marius, "Konto 1"));
    bank1->addAccount(new BankAccount(marius, "Konto 2"));
    bank3->addAccount(new BankAccount(marius, "Konto 3"));

    banks.insert(bank1);
    banks.insert(bank2);
    banks.insert(bank3);

    auto * cashFactory = new CashFactory();

    for(Bank * bank : banks) {
        std::cout << std::endl;
        std::cout << "======================== " << bank->getName() << " ========================" << std::endl;
        std::cout << std::endl;
        for(BankAccount * bankAccount : bank->getAccounts()) {
            for (int i = 0; i < 10000; ++i) {
                int rnd = distribution(generator);
                Bill * bill = cashFactory->printBill(rnd, CurrencyValue::USD);
                Coin * coin = cashFactory->printCoin(rnd, CurrencyValue::USD);
                if(bill != nullptr) {
                    *bankAccount + bill;
                }
                if(coin != nullptr) {
                    *bankAccount + coin;
                }

                Money * fee = *bankAccount - 10;
                delete fee;
            }
            std::cout << *bankAccount << std::endl;
        }
    }

    for (Bank *bank : banks) {
        for (BankAccount *bankAccount : bank->getAccounts()) {
            delete(bankAccount);
        }
        delete(bank);
    }

}