#include <algorithm>
#include "CashFactory.h"

using namespace banking;

int allowedBills[5] = {100, 50, 20, 10, 5};
int allowedCoins[8] = {1, 2, 5, 10, 20, 50, 100, 200};

std::vector<std::string> usedSerials;

std::shared_ptr<Bill> CashFactory::printBill(int value, Currency currency) {
    auto * iter = std::find(std::begin(allowedBills), std::end(allowedBills), value);
    if(iter == std::end(allowedBills)) {
        return nullptr;
    }else {
        std::string serial = randomString(15);
        while(std::find(usedSerials.begin(), usedSerials.end(), serial) != usedSerials.end()) {
            serial = randomString(15);
        }
        usedSerials.push_back(serial);
        return std::make_shared<Bill>(value, currency, serial);
    }
}

std::shared_ptr<Coin> CashFactory::printCoin(int value, Currency currency) {
    auto * iter = std::find(std::begin(allowedCoins), std::end(allowedCoins), value);
    if(iter == std::end(allowedCoins)) {
        return nullptr;
    }else return std::make_shared<Coin>(value, currency);
}

std::string CashFactory::randomString(size_t length) {
    auto randchar = []() -> char {
        const char charset[] =
                "0123456789"
                "ABCDEFGHIKLMNOPRSTUVWXYZ"
                "abcdfghijklmnoqrstuvwxyz";
        const size_t max_index = (sizeof(charset) - 1);
        return charset[rand() % max_index];
    };
    std::string str(length, 0);
    std::generate_n(str.begin(), length, randchar);
    return str;

}